@@ -1,7 +1,7 @@
 #include "GBase.h"
 #include <stdarg.h>
 #include <ctype.h>
-#include <sys/stat.h>
+#include <errno.h>
 
 #ifndef S_ISDIR
 #define S_ISDIR(mode)  (((mode) & S_IFMT) == S_IFDIR)
@@ -56,8 +56,13 @@ void GAssert(const char* expression, const char* filename, unsigned int lineno){
   char msg[4096];
   sprintf(msg,"%s(%d): ASSERT(%s) failed.\n",filename,lineno,expression);
   fprintf(stderr,"%s",msg);
-  //abort();
-  }
+  #ifdef DEBUG
+  // modify here if you [don't] want a core dump
+    abort();
+  #endif
+  exit(1);
+}
+
 // Error routine (prints error message and exits!)
 void GError(const char* format,...){
   #ifdef __WIN32__
@@ -166,7 +171,7 @@ char* newEmptyStr() {
 char* Gstrdup(const char* sfrom, const char* sto) {
   if (sfrom==NULL || sto==NULL) return NULL;
   char *copy=NULL;
-  if (sfrom[0]==0) return newEmptyStr();
+  if (sfrom[0]==0 || sto<sfrom) return newEmptyStr();
   GMALLOC(copy, sto-sfrom+2);
   strncpy(copy, sfrom, sto-sfrom+1);
   copy[sto-sfrom+1]=0;
@@ -184,6 +189,103 @@ int Gstrcmp(const char* a, const char* b, int n) {
 
 }
 
+int G_mkdir(const char* path, int perms = (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) ) {
+   //int perms=(S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) ) {
+ #ifdef __WIN32__
+     return _mkdir(path);
+ #else
+     return  mkdir(path, perms);
+ #endif
+}
+
+
+void Gmktempdir(char* templ) {
+#ifdef __WIN32__
+  int blen=strlen(templ);
+  if (_mktemp_s(templ, blen)!=0)
+	  GError("Error creating temp dir %s!\n", templ);
+#else
+  char* cdir=mkdtemp(templ);
+  if (cdir==NULL)
+	  GError("Error creating temp dir %s!(%s)\n", templ, strerror(errno));
+#endif
+}
+
+int Gmkdir(const char *path, bool recursive, int perms) {
+	if (path==NULL || path[0]==0) return -1;
+	mode_t process_mask = umask(0); //is this really needed?
+	if (!recursive) {
+	   int r=G_mkdir(path, perms);
+	   if (r!=0) 
+	      GMessage("Warning: G_mkdir(%s) failed: %s\n", path, strerror(errno));
+	   umask(process_mask);
+	   return r;
+	   }
+	int plen=strlen(path);
+	char* gpath=NULL;
+	//make sure gpath ends with /
+	if (path[plen-1]=='/') {
+		gpath=Gstrdup(path);
+	}
+	else {
+		GMALLOC(gpath, plen+2);
+		strcpy(gpath,path);
+		gpath[plen]='/';
+		gpath[plen+1]=0;
+	}
+	char* ss=gpath;
+	char* psep = NULL;
+	while (*ss!=0 && (psep=strchr(ss, '/'))!=NULL)  {
+		*psep=0; //now gpath is the path up to this /
+		ss=psep; ++ss; //ss repositioned just after the /
+		// create current level if it doesn't exist
+		if (fileExists(gpath)) { //path exists
+		    *psep='/';
+		    continue; //assume it's a directory or a symlink to one
+		              //if not, it'll fail later
+		}
+		int mkdir_err=0;
+		if ((mkdir_err=G_mkdir(gpath, perms))!=0) {
+				GMessage("Warning: mkdir(%s) failed: %s\n", gpath, strerror(errno));
+			GFREE(gpath);
+			umask(process_mask);
+			return -1;
+		}
+		*psep='/';
+	}
+	GFREE(gpath);
+	umask(process_mask);
+	return 0;
+}
+
+FILE* Gfopen(const char *path, char *mode) {
+	FILE* f=NULL;
+	if (mode==NULL) f=fopen(path, "rb");
+	    	   else f=fopen(path, mode);
+	if (f==NULL)
+		GMessage("Error opening file '%s':  %s\n", path, strerror(errno));
+	return f;
+}
+
+bool GstrEq(const char* a, const char* b) {
+	 if (a==NULL || b==NULL) return false;
+	 register int i=0;
+	 while (a[i]==b[i]) {
+		 if (a[i]==0) return true;
+		 ++i;
+	 }
+	 return false;
+}
+
+bool GstriEq(const char* a, const char* b) {
+	 if (a==NULL || b==NULL) return false;
+	 register int i=0;
+	 while (tolower((unsigned char)a[i])==tolower((unsigned char)b[i])) {
+		 if (a[i]==0) return true;
+	 }
+	 return false;
+}
+
 int Gstricmp(const char* a, const char* b, int n) {
  if (a==NULL || b==NULL) return a==NULL ? -1 : 1;
  register int ua, ub;
@@ -492,7 +594,7 @@ char* strifind(const char* str,  const char* substr) {
   char* p=(char*)str;
   while (p<=smax) {
      for (i=0; i<l && tolower(*(p+i))==tolower(*(substr+i)); i++) ;
-     if (i==l) return p; //found!
+     if (i==l) return p;
      p++;
      }
   return NULL;
@@ -508,6 +610,14 @@ bool startsWith(const char* s, const char* prefix) {
  return (prefix[i]=='\0');
  }
 
+bool startsiWith(const char* s, const char* prefix) {
+ if (prefix==NULL || s==NULL) return false;
+ int i=0;
+ while (prefix[i]!='\0' && tolower(prefix[i])==tolower(s[i])) i++;
+ return (prefix[i]=='\0');
+ }
+
+
 // tests if string s ends with given suffix
 bool endsWith(const char* s, const char* suffix) {
  if (suffix==NULL || s==NULL) return false;
@@ -599,12 +709,12 @@ const char* getFileExt(const char* filepath) {
 int fileExists(const char* fname) {
   struct stat stFileInfo;
   int r=0;
-  // Attempt to get the file attributes
+  // Attempt to get the path attributes
   int fs = stat(fname,&stFileInfo);
   if (fs == 0) {
       r=3;
       // We were able to get the file attributes
-      // so the file obviously exists.
+      // so the path exists
       if (S_ISREG (stFileInfo.st_mode)) {
          r=2;
          }
@@ -615,14 +725,6 @@ int fileExists(const char* fname) {
   return r;
 }
 
-/*bool fileExists(const char* filepath) {
-  if (filepath==NULL) return false;
-  FILE* ft=fopen(filepath, "rb");
-  if (ft==NULL) return false;
-  fclose(ft);
-  return true;
-}
-*/
 int64 fileSize(const char* fpath) {
   struct stat results;
   if (stat(fpath, &results) == 0)
@@ -773,5 +875,5 @@ char* commaprintnum(uint64 n) {
     n /= 10;
     i++;
   } while(n != 0);
-  return p;
+  return Gstrdup(p);
 }
